@@ -13,7 +13,7 @@
 #    You should have received a copy of the GNU Lesser General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-__version__ = '0.8.1'
+__version__ = '0.8.2'
 
 import os
 import warnings
@@ -22,9 +22,9 @@ from new import classobj
 
 __all__ = ['ArgvalidateException', 'DecoratorLengthException',
     'DecoratorNonKeyLengthException', 'DecoratorKeyUnspecifiedException',
-    'ArgumentTypeException', 'ArgumentLengthException', 'func_args',
-    'method_args', 'return_value', 'one_of', 'type_any', 'raises_exceptions',
-    'warns_kwarg_as_arg']
+    'DecoratorStackingException', 'ArgumentTypeException',
+    'ArgumentLengthException', 'func_args', 'method_args', 'return_value',
+    'one_of', 'type_any', 'raises_exceptions', 'warns_kwarg_as_arg']
 
 # Check for environment variables
 argvalidate_warn = 0
@@ -53,7 +53,7 @@ class ArgvalidateErrorException(Exception):
 
     .. note::
 
-       Do not use this warning directly. Always use ArgvalidateException
+       Do not use this exception directly. Always use ArgvalidateException
        when checking for argvalidate errors.
 
     """
@@ -100,7 +100,7 @@ class ArgumentLengthException(ArgvalidateException):
         self.expected_count = expected_count
         self.passed_count = passed_count
 
-        msg = '%s: too few arguments passed to function (expected %d, got %d).' %\
+        msg = '%s: wrong number of arguments passed to function (expected %d, got %d).' %\
             (func_name, expected_count, passed_count)
         ArgvalidateException.__init__(self, msg)
 
@@ -125,7 +125,7 @@ class DecoratorLengthException(ArgvalidateException):
         self.func_name = func_name
         self.expected_count = expected_count
         self.passed_count = passed_count
-        msg = '%s: too few arguments specified in decorator ' %\
+        msg = '%s: wrong number of arguments specified in decorator ' %\
             (func_name)
         msg += '(expected %d, got %d).' % (expected_count, passed_count)
         ArgvalidateException.__init__(self, msg)
@@ -179,6 +179,27 @@ class DecoratorKeyUnspecifiedException(ArgvalidateException):
             (func_name, arg_name)
         ArgvalidateException.__init__(self, msg)
 
+class DecoratorStackingException(ArgvalidateException):
+    """
+    Exception for stacking a decorator with itself.
+
+    This exception provides the following attributes:
+
+    * func_name
+        Name of function that caused the exception to be raised
+        (str, read-only).
+
+    * decorator_name
+        Name of the decorator that was stacked with itself (str, read-only).
+
+    """
+    def __init__(self, func_name, decorator_name):
+        self.func_name = func_name
+        self.decorator_name = decorator_name
+        msg = '%s: decorator %s stacked with itself.' %\
+            (func_name, decorator_name)
+        ArgvalidateException.__init__(self, msg)
+
 class ArgumentTypeException(ArgvalidateException):
     """
     Exception for invalid argument type.
@@ -290,7 +311,21 @@ def __check_type(func_name, arg_name, expected_type, passed_value,\
             passed_type), stacklevel=stacklevel)
 
 def __check_args(type_args, type_kwargs, start=0):
-    def validate(func):
+    def validate(f):
+        args_func = getattr(f, '_args_stacked_func', None)
+        if args_func:
+            if start == 0:
+                raise DecoratorStackingException(args_func.func_name,\
+                     'func_args')
+            elif start == 1:
+                raise DecoratorStackingException(args_func.func_name,\
+                     'method_args')
+            else:
+                raise DecoratorStackingException(args_func.func_name,\
+                     'unknown; start=%d' % (start))
+
+        func = getattr(f, '_return_value_stacked_func', f)
+
         func_argcount = func.func_code.co_argcount - start
         func_key_argcount = 0
         f_name = func.func_name
@@ -361,6 +396,7 @@ def __check_args(type_args, type_kwargs, start=0):
             return func(*call_args, **call_kwargs)
 
         __wrapper_func.func_name = func.func_name
+        __wrapper_func._args_stacked_func = func
         return __wrapper_func
     
     return validate
@@ -403,7 +439,7 @@ def func_args(*type_args, **type_kwargs):
          pass
          
     """
-    return __check_args(type_args, type_kwargs)
+    return __check_args(type_args, type_kwargs, start=0)
 
 def method_args(*type_args, **type_kwargs):
     """
@@ -461,17 +497,25 @@ def return_value(expected_type):
             return 5
             
     """
-    def validate(func):
+    def validate(f):
+        return_value_func = getattr(f, '_return_value_stacked_func', None)
+        if return_value_func:
+            raise DecoratorStackingException(return_value_func.func_name,\
+                'return_value')
+
+        func = getattr(f, '_args_stacked_func', f)
+
         def __wrapper_func(*args, **kwargs):
             result = func(*args, **kwargs)
             __check_return_value(func.func_name, expected_type, result)
             return result
         
         __wrapper_func.func_name = func.func_name
+        __wrapper_func._return_value_stacked_func = func
         return __wrapper_func
     return validate
 
-class OneOfTuple(tuple):
+class __OneOfTuple(tuple):
     def __repr__(self):
         return 'one of %r' % (tuple.__repr__(self))
 
@@ -493,7 +537,7 @@ def one_of(*args):
             pass
             
     """
-    return OneOfTuple(args)
+    return __OneOfTuple(args)
 
 def raises_exceptions():
     """
