@@ -25,6 +25,7 @@ from argvalidate import ArgumentTypeException, ArgumentLengthException
 from argvalidate import ReturnValueTypeException, DecoratorLengthException
 from argvalidate import DecoratorNonKeyLengthException
 from argvalidate import DecoratorKeyUnspecifiedException
+from argvalidate import DecoratorStackingException
 
 loader = unittest.TestLoader()
 
@@ -284,6 +285,80 @@ class ArgvalidateReturnValueTestCase(unittest.TestCase):
 ArgvalidateReturnValueSuite =\
      loader.loadTestsFromTestCase(ArgvalidateReturnValueTestCase)
 
+class ArgvalidateStackingCase(unittest.TestCase):
+    def test00_stacking_return_value_first(self):
+        try:
+            @return_value(int)
+            @func_args(str)
+            def test_func(data):
+                return 1
+
+            class A(object):
+                @return_value(int)
+                @method_args(str)
+                def test_func(self, data):
+                    return 1
+
+            test_func('test')
+            A().test_func('test')
+        except DecoratorStackingException, e:
+            self.fail('DecoratorStackingException raised: %s' % (e.message))
+
+    def test01_stacking_args_first(self):
+        try:
+            @func_args(str)
+            @return_value(int)
+            def test_func(data):
+                return 1
+
+            class A(object):
+                @method_args(str)
+                @return_value(int)
+                def test_func(self, data):
+                    return 1
+
+            test_func('test')
+            A().test_func('test')
+        except DecoratorStackingException, e:
+            self.fail('DecoratorStackingException raised: %s' % (e.message))
+
+    def test02_stacking_args_invalid(self):
+        try:
+            @func_args(str)
+            @func_args(str)
+            def test_func(data):
+                return data
+
+            self.fail('DecoratorStackingException not raised for double'+\
+                 ' func_args decorator.')
+        except DecoratorStackingException:
+            pass
+
+        try:
+            class A(object):
+                @method_args(str)
+                @method_args(str)
+                def test_func(self, data):
+                    return data
+
+            self.fail('DecoratorStackingException not raised for double'+\
+                ' method_args decorator.')
+        except DecoratorStackingException:
+            pass
+
+    def test03_stacking_return_value_invalid(self):
+        try:
+            @return_value(int)
+            @return_value(int)
+            def test_func():
+                return 5
+            self.fail('DecoratorStackingException not raised for double'+\
+                ' return_value decorator.')
+        except DecoratorStackingException:
+            pass
+
+ArgvalidateStackingSuite = loader.loadTestsFromTestCase(ArgvalidateStackingCase)
+
 ArgvalidateTestSuite = unittest.TestSuite(
   [ArgvalidateFuncSuite, ArgvalidateMethodSuite, ArgvalidateReturnValueSuite,
-  ArgvalidateLengthSuite])
+  ArgvalidateLengthSuite, ArgvalidateStackingSuite])
